/*	$Csoft	    */

/*
 * Copyright (c) 1999, 2000, 2001 CubeSoft Communications, Inc.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistribution of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistribution in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 * 3. Neither the name of CubeSoft Communications, nor the names of its
 *    contributors may be used to endorse or promote products derived from
 *    this software without specific prior written permission.
 * 
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED. IN NO EVENT SHALL THE AUTHOR OR CONTRIBUTORS BE LIABLE FOR
 * ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR
 * SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER
 * CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
 * OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE
 * USE OF THIS SOFTWARE EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */

#include <sys/types.h>
#include <sys/mman.h>
#include <stdlib.h>
#include <stdio.h>
#include <unistd.h>
#include <fcntl.h>
#include <glib.h>

#include "techmlm.h"

char 
techmlm_deliver(struct techmlm *ti)
{
	int f;
	size_t fl;
	char *s, **w, **wp, *cw, *dcmd;

	f = open(ti->lf, O_RDONLY, 00600);
	cw = NULL;

	if (f < 0) {
		log(ti, L_ERROR, "cannot open %s", ti->lf);
		return (D_FATAL);
	}
	fl = lseek(f, 0, SEEK_END);
	if (fl < 3) {
		return (D_FATAL);
	}

	s = mmap(0, fl, PROT_READ, MAP_PRIVATE, f, 0);
	if ((void *)s == MAP_FAILED) {
		perror("mmap");
		return (D_FATAL);
	}
	close(f);

	w = g_strsplit(s, "\n", 0);

	if (ti->email) {
		wp = w;
		while ((cw = *wp++)) {
			if (!strcmp(cw, ti->email)) {
				log(ti, L_ERROR, "circular reference for %s",
				    ti->email);
				return (D_FAIL);
			}
		}
	}

	cw = g_strjoinv(" ", w);
	dcmd = g_strdup_printf("%s %s |%s %s", CAT_CMD, ti->tf,
	    DEFAULT_MTA, cw);
	system(dcmd);

	free(dcmd);
	free(cw);
	g_strfreev(w);
	munmap(s, fl);

	return (D_OK);
}
